#include <iostream.h>
#include <math.h>

#include  "autojust\matrix.h"


//******************************************************************************
// Matrix-Klasse
//******************************************************************************

// *** Klassenkonstruktoren ***************************************************
// Standardkonstruktor (ohne Argumente)
TMatrix::TMatrix()
{
  arr = NULL;
  ze = sp = 0;
  homogene_koordinaten=false;
}

// Konstruktor, der eine m*n-Nullmatrix initialisiert.
TMatrix::TMatrix(unsigned m, unsigned n)
{
  ze = m; sp = n;
  homogene_koordinaten=false;
  arr = new double[m*n]; // dynamisches Erzeugen des Matrixarrays
  for (unsigned i=0; i < m*n; i++) arr[i] = 0; // Array mit Nullen belegen
}

// Klassendestruktor lscht das dynamische Array
TMatrix::~TMatrix()
{
  if (arr != NULL) delete [] arr;
}

// berladener copy-Kontruktor (wird nur bei Initialisierung aufgerufen)
TMatrix::TMatrix (const TMatrix & mat)
{
  ze = mat.ze;
  sp = mat.sp;
  homogene_koordinaten=mat.homogene_koordinaten;
  arr = new double[mat.ze*mat.sp];
  for (unsigned i=0; i < mat.ze * mat.sp; i++) arr[i] = mat.arr[i];
}

// *** berladene Operatoren *************************************************
// berladener = Operator : Zuweisungsoperator
TMatrix TMatrix::operator = (const TMatrix & mat)
{
  if (this == &mat) return *this; // *this zurckgeben, wenn Selbstzuweisung

  if (arr != NULL) delete [] arr; // Speicherplatz des Matrix-Arrays freigeben
  ze = mat.ze;
  sp = mat.sp;
  homogene_koordinaten= mat.homogene_koordinaten;
  arr = new double[mat.ze*mat.sp];
  for (unsigned i=0; i < mat.ze * mat.sp; i++) arr[i] = mat.arr[i];
  return *this;
}

// berladener + Operator: addiert zwei Matrizen gleichen Typs
TMatrix TMatrix::operator + (const TMatrix & mat)
{
  if (ze != mat.ze || sp != mat.sp)
  {
    cerr << "(M.+)Fehler: Matrizen versch.Typs knnen nicht addiert werden.\n";
    return *this;
  }
  else
  {
  for (unsigned i=0; i < ze*sp; i++)
    arr[i] += mat.arr[i];
  }
  homogene_koordinaten &= mat.homogene_koordinaten;
  return *this;
}

// berladener - Operator: subtrahiert zwei Matrizen gleichen Typs
TMatrix TMatrix::operator - (const TMatrix & mat)
{
  if (ze != mat.ze || sp != mat.sp)
  {
    cerr << "(M.-)Fehler: Kann Matrizen versch. Typs nicht subtrahieren.\n";
    return *this;
  }
  else
  {
    for (unsigned i=0; i < ze*sp; i++)
      arr[i] -= mat.arr[i];
  }
  homogene_koordinaten &= mat.homogene_koordinaten;
  return *this;
}

// berladener * Operator Nr.1: reelle Vervielfachung von Matrizen
// Version TMatrix-Objekt * double-Wert
TMatrix TMatrix::operator * (double & fakt)
{
  for (unsigned i=0; i < ze*sp; i++)
    arr[i] *= fakt;
  return *this;
}

// Version double-Wert * TMatrix-Objekt (friend-Funktion)
TMatrix operator * (double fakt, const TMatrix & mat)
{
  TMatrix temp(mat.ze, mat.sp);
  for (unsigned i=0; i < mat.ze*mat.sp; i++)
    temp.arr[i] = fakt * mat.arr[i];
  temp.homogene_koordinaten=mat.homogene_koordinaten;
  return temp;
}

// berladener * Operator Nr.2: multipliziert zwei Matrizen
TMatrix TMatrix::operator * (const TMatrix & mat)
{
  TMatrix temp(ze,mat.sp);
  if (sp != mat.ze)
  {
    cerr << "(M.*)Fehler: Spalten der 1. Matrix ungleich Zeilen der 2.\n";
    temp.ze = temp.sp = 0;
    temp.arr = NULL;
  }
  else
  {
    for (unsigned i=0; i < ze*mat.sp; i++)
      for (unsigned j=0; j < sp; j++)
        temp.arr[i] += arr[j + sp*(i/mat.sp)] * mat.arr[j*mat.sp + i % mat.sp];
  }
  temp.homogene_koordinaten=(homogene_koordinaten && mat.homogene_koordinaten);
  return temp;
}

// *** Matrizen - Funktionen **************************************************
// Erstellen einer Einheitsmatrix mit Rang m
TMatrix TMatrix::einheitsmatrix(unsigned m)
{
  if (arr != NULL) delete [] arr;

  ze = m;
  sp = m;

  arr = new double[m*m];

  for (unsigned i=0; i < ze; i++)
  {
    for (unsigned j=0; j < sp; j++)
    {
      if ( i == j) arr[i*sp + j] = 1;
      else arr[i*sp + j] = 0;
    }
  }
  return *this;
}

// Berechnung der Inversen einer Matrix, sofern sie existiert
// Verfahren von Gauss-Jordan (siehe Stoecker S.402)
TMatrix TMatrix::invers(void)
{
  double dummy;

  if ( ze != sp || ze == 0 || sp == 0)
    cerr << "(M.Inv)Fehler: Matrix kann nicht invertiert werden.\n";

  TMatrix temp;

  // Einheitsmatrix temporr erstellen
  temp.einheitsmatrix(ze);

  //Berechnung der Inversen
  for (unsigned k=0; k < ze; k++)
  {
    dummy = arr[k*sp + k];
    for (unsigned j=0; j < sp; j++)
    {
      arr[k*sp + j] = arr[k*sp +j]/dummy;
      temp.arr[k*sp + j] = temp.arr[k*sp +j]/dummy;
    }
    for (unsigned i=0; i < ze; i++)
    {
      if (i != k)
      {
        dummy = arr[i*sp + k];
        for (unsigned j=0; j < sp; j++)
        {
          arr[i*sp + j] = arr[i*sp + j]-(dummy*arr[k*sp +j]);
          temp.arr[i*sp + j] = temp.arr[i*sp + j]-(dummy*temp.arr[k*sp +j]);
        }
      }
    }
  }
  // Inverse Matrix ist nun in temp
  return temp;
}

// **** Transformationsmethoden ***********************************************
// Verschiebe-(Translations)TMatrix erzeugen
// Verschiebung um (Tx,Ty,Tz) Vektor
TMatrix TMatrix::verschiebematrix(const TVektor &vekt)
{
  if ( !(vekt.ze == 4 && vekt.ist_homogen() ||
         vekt.ze == 3 && !vekt.ist_homogen()))
  {
    cerr << "(M.VM)Fehler: Verschiebungsvektor nicht  3-dimensional.\n";
    return *this;
  }
  // Einheitsmatrix 4x4 erzeugen
  this->einheitsmatrix(4);
  // Verschiebungsvektor in die letzte Spalte der Matrix eintragen
  // bei homogenem Vektor wird homogene Komponente mitkopiert (=1)
  for (unsigned i = 0; i < vekt.ze; i++)
  {
    // Zugriff auf 4. Spaltenelement der jeweiligen Zeile
    arr[i*ze+3] = vekt.arr[i];
  }
  homogene_koordinaten=true;
  return *this;
}

// Rotationsmatrix erzeugen
// Drehung um x-,y-, bzw. z-Achse mit Winkel
// Anmerkung: wenn Winkel negativ angegeben, dann handelt es sich um eine
//            inverse Drehung
TMatrix TMatrix::rotationsmatrix_x(double winkel)
{
  this->einheitsmatrix(4);
  /* Rotation um die X-Achse mit dem Winkel a
       1   0       0     0
       0 cos(a) -sin(a)  0
       0 sin(a)  cos(a)  0
       0   0       0     1
  */
  arr[1*4+1]= cos(winkel); arr[1*4+2]= -sin(winkel);
  arr[2*4+1]= sin(winkel); arr[2*4+2]=  cos(winkel);
  homogene_koordinaten=true;
  return *this;
}

TMatrix TMatrix::rotationsmatrix_y(double winkel)
{
  this->einheitsmatrix(4);
  /* Rotation um die Y-Achse mit dem Winkel a
     cos(a)  0   sin(a)  0
       0     1     0     0
    -sin(a)  0   cos(a)  0
       0     0     0     1
  */
  arr[0*4+0]=  cos(winkel); arr[0*4+2]= sin(winkel);
  arr[2*4+0]= -sin(winkel); arr[2*4+2]= cos(winkel);
  homogene_koordinaten=true;
  return *this;
}

TMatrix TMatrix::rotationsmatrix_z(double winkel)
{
  this->einheitsmatrix(4);
  /* Rotation um die z-Achse mit dem Winkel a
    cos(a) -sin(a)  0   0
    sin(a)  cos(a)  0   0
      0       0     1   0
      0       0     0   1
  */
  arr[0*4+0]= cos(winkel); arr[0*4+1]= -sin(winkel);
  arr[1*4+0]= sin(winkel); arr[1*4+1]=  cos(winkel);
  homogene_koordinaten=true;
  return *this;
}

// Zusammenfassung aller Transformationen X-Y-Z & Verschiebung in einer Funktion
TMatrix TMatrix::transformiere(unsigned reihenfolge, \
                           const TVektor & verschiebung, \
                           double drehung_x, double drehung_y, double drehung_z)
{
  TMatrix tempx, tempy, tempz;

  // Nur 4dim Vektoren inkl. homogener Komponente oder reine 3dim Vektoren
  if ( !(verschiebung.ze == 4 && verschiebung.ist_homogen() || \
         verschiebung.ze == 3 && !verschiebung.ist_homogen()) )
  {
    cerr << "(M.TRAFO)Fehler: Verschiebungsvektor nicht  3-dimensional.\n";
    return *this;
  }

  // Reihenfolge der Transformationen muss angegeben sein: X->Y->Z oder Z->Y->X
  if ( reihenfolge != REIHENFOLGE_XYZ && reihenfolge != REIHENFOLGE_ZYX )
  {
    cerr << "(M.TRAFO)Fehler: Keine gltige Angabe der Trafo-Reihenfolge.\n";
    return *this;
  }
  else
  {
    // Drehungen in der 4x4 Trafo-Matrix
    // Erklrung siehe oben bei Einzelrotationen
    tempx.rotationsmatrix_x(drehung_x);
    tempy.rotationsmatrix_y(drehung_y);
    tempz.rotationsmatrix_z(drehung_z);
    // Verschiebung um Vektor
    this->verschiebematrix(verschiebung);
    if  (reihenfolge == REIHENFOLGE_ZYX)
      // aufgrund der Def. d. Matrixmultiplikation (Beginn der Mult. von rechts)
      // muss die Reihenfolge der Multiplikation umgekehrt notiert werden,
      // mathematisch gilt folgendes:
      // R_rueck = Rx*Ry*Rz*T
      *this = *this * tempz * tempy * tempx;
    else
      // aufgrund der Def. d. Matrixmultiplikation (Beginn der Mult. von rechts)
      // muss die Reihenfolge der Multiplikation umgekehrt notiert werden,
      // mathematisch gilt folgendes:
      // R_hin = T*Rz*Ry*Rx
      *this = tempx * tempy * tempz * *this;
    homogene_koordinaten=true;
  }
  return *this;
}


//******************************************************************************
// Vektor-Klasse
//******************************************************************************

// Funktion, um aus einer (m,1)-Matrix einen Vektortyp zu machen
TVektor::TVektor (const TMatrix & mat)
{

  if (mat.ze == 0 || mat.sp == 0)
    cerr << "(V)Fehler: Matrixspalten- oder Zeilenanzahl betrgt 0.\n";
  else if (mat.sp != 1)
    cerr << "(V)Fehler: Matrixspaltenanzahl != 1 Keine Umwandlung in Vektor.\n";
  else
  {
    ze=mat.ze;
    sp=1;
    homogene_koordinaten=mat.homogene_koordinaten;
    arr = new double[ze];
    for (unsigned i=0; i < ze; i++) arr[i] = mat.arr[i];
  }
}


// Vektor skalieren
TVektor operator * (double fakt, const TVektor & vekt)
{
  TVektor temp(vekt.ze);
  for (unsigned i=0; i < vekt.ze; i++)
    temp.arr[i] = fakt * vekt.arr[i];
  return temp;
}

// Umwandlung in einen Vektor in homogenen Koordinaten
TVektor TVektor::mache_homogen(void)
{
  unsigned i;

  if (ze == 0)
  {
    cerr << "(V.MH)Fehler: Vektor mit Zeilenanzahl 0.\n";
    return *this;
  }
  else if (this->ist_homogen())
  {
	 cerr << "(V.MH)Fehler: Vektor schon in homog. Koord. angegeben.\n";
    return *this;
  }

  TVektor temp(ze+1); // temp-Vektor mit einer zustzlichen homogenen Komponente

    // Kopieren des Ursprungsvektors
  for (i=0; i < ze; i++) temp.arr[i] = arr[i];
  // neue Komponente im homogenen temp-Vektor auf 1 setzen
  temp.arr[(ze+1)-1] = 1;

  // aktuellen Vektor um homogene Komponente erweitern
  if (arr != NULL) delete [] arr; // Speicherplatz des Matrix-Arrays freigeben
  ze = ze+1;
  arr = new double[ze];           // neues Vektor-Array erzeugen
  for (i=0; i < ze; i++) arr[i] = temp.arr[i];
  homogene_koordinaten = true;
  return *this;
}

// Umwandlung in einen Vektor in kartesischen Koordinaten
TVektor TVektor::mache_kartesisch(void)
{
  unsigned i;

  if (ze == 0)
  {
    cerr << "(V.MK)Fehler: Vektor mit Zeilenanzahl 0.\n";
    return *this;
  }
  else if (!this->ist_homogen())
  {
	 cerr << "(V.MK)Fehler: Vektor bereits in kartes. Koord. angegeben.\n";
    return *this;
  }

  TVektor temp(ze-1); // temp-Vektor ohne homogene Komponente

  // Kopieren des Ursprungsvektors
  // aktueller Vektor ohne homogene Komponente
  for (i=0; i < temp.ze; i++) temp.arr[i] = arr[i];

  if (arr != NULL) delete [] arr; // Speicherplatz des Vektor-Arrays freigeben
  ze = ze-1;
  arr = new double[ze];           // neues Vektor-Array erzeugen
  for (i=0; i < ze; i++) arr[i] = temp.arr[i];
  homogene_koordinaten = false;
  return *this;
}

// Berechnung des Betrags (Lnge) eines Vektors
double TVektor::vektor_betrag (void) const
{
  double betrag=0.0;
  unsigned temp_ze;

  if (ze == 0)
  {
    cerr << "(V.VB)Fehler: Kein echter Vektor (Zeilen == 0).\n";
    return -1.0;
  }

  if (this->ist_homogen())
    temp_ze=ze-1; // homogene Komponente des Vektors ignorieren
  else
    temp_ze=ze;   // alle kartesischen Koordinaten betrachten

  for (unsigned i = 0; i < temp_ze; i++)
  {
    betrag += arr[i]*arr[i];
  }
  return sqrt(betrag);
}

// Berechnung des Skalarproduktes zweier Vektoren
double TVektor::skalarprodukt (const TVektor & vekt)
{
  double skalar=0.0;
  unsigned temp_ze;

  if (ze == 0 || vekt.ze == 0)
  {
    cerr << "(V.SP)Fehler: Mindestens ein Vektor mit Dimension 0.\n";
    return -1.0;
  }
  else if (ze != vekt.ze)
  {
    cerr << "(V.SP)Fehler: Nur Vektoren gleicher Dimension zulssig.\n";
    return -1.0;
  }

  if (this->ist_homogen() && vekt.ist_homogen())
    temp_ze=ze-1; // homogene Komponente des Vektors ignorieren
  else if (!(this->ist_homogen()) && !(vekt.ist_homogen()))
    temp_ze=ze;   // alle kartesischen Koordinaten betrachten
  else
  {
	 cerr << "(V.SP)Fehler: 1 Vektor ist in homog. Koord. angegeben.\n";
    return -1.0;
  }

  for (unsigned i = 0; i < temp_ze; i++)
  {
    skalar += arr[i]*vekt.arr[i];
  }
  return skalar;
}

// Berechnung des Winkels zwischen zwei Vektoren in RADIANT
double TVektor::winkel (const TVektor & vekt)
{
  double winkel=-1.0; // falls Rckgabewert -1.0, dann Fehlerfall

  if (ze == 0 || vekt.ze == 0)
  {
    cerr << "(V.Wi)Fehler: Mindestens ein Vektor mit Dimension 0.\n";
    return winkel;
  }
  else if (ze != vekt.ze)
  {
    cerr << "(V.Wi)Fehler: Nur Vektoren gleicher Dimension zulssig.\n";
    return winkel;
  }
  else if ( (this->ist_homogen() && !(vekt.ist_homogen()) ) ||
            (!(this->ist_homogen()) && vekt.ist_homogen() )    )
  {
    cerr << "(V.Wi)Fehler: Nur 1 Vektor in homogenen Koordinaten angegeben.\n";
    return winkel;
  }

  // Berechnung von |a|*|b|
  if ((winkel = vekt.vektor_betrag() * this->vektor_betrag()) == 0)
  {
    cerr << "(V.Wi)Fehler: Winkel kann nicht berechnet werden (Nullvektor).\n";
    return winkel;
  }
  // Berechnung von a * b / |a|*|b|
  winkel = this->skalarprodukt(vekt) / winkel;
  // Berechnung von arcos (a*b / |a|*|b|) = Winkel zw. 2 Vektoren
  winkel = acos(winkel);
  return winkel;
}

// Setzen der x,y,z-Koordinaten eines 3dim Vektors
bool TVektor::set_XYZ(double x, double y, double z)
{
  if (ze == 3)
  {
    arr[0]=x; arr[1]=y; arr[2]=z;
    return true;
  }
  else return false;
}

// Auslesen der x,y,z-Koordinaten eines 3dim Vektors
bool TVektor::get_XYZ(double & x, double & y, double & z)
{
  if ((ze == 3) || (ze == 4))
  {
    x=arr[0]; y=arr[1]; z=arr[2];
    return true;
  }
  else return false;
}


//******************************************************************************
// Matrizenliste-Klasse
//******************************************************************************
// Liste von Transformationsmatrizen
// organisiert als Stack
// mit push(element), pop(), ist_leer()


// Listenkonstruktor um eine Liste mit einer bestimmten Anzahl von
// Listenelementen zu erzeugen
TMatrizenListe::TMatrizenListe(unsigned anzahl)
{
  liste = NULL;
  akt_elemente = 0;
  max_elemente = anzahl;

  if (anzahl != 0 )
  {
    liste = new TMatrix[max_elemente];
  }
}

// Matrix zur Liste hinzufgen
bool TMatrizenListe::push(const TMatrix & trafo)
{
  if (akt_elemente != max_elemente)
  {
    liste[akt_elemente++] = trafo;
    return true;
  }

  return false;
}

// letzte Matrix aus Liste entfernen
TMatrix TMatrizenListe::pop(void)
{
  TMatrix temp;

  if (akt_elemente)
  {
    temp = liste[--akt_elemente];
  }
  return temp;
}

TMatrix TMatrizenListe::zeige(unsigned position)
{
  TMatrix temp;

  if (position == 0)
  {
    cerr << "(ML.Zeig)Fehler: Nulltes Listenelement existiert nicht.\n";
  }
  else if (akt_elemente && (akt_elemente >= position) )
  {
    temp = liste[position-1];
  }

  return temp;
}


