#ifndef _WORKFLOW_H_
#define _WORKFLOW_H_


#define   nMaxArg   4

// Commands
typedef enum 
{ 
  GotoPeak = 1, GotoIntensity, ChooseAxis, GetHWB, SetWidth, 
      AreaScan, Scan, ControlFlank, ShowHWB, SaveData, ChooseDevice, 
      Calculate, LoadPoint, ShowValue, MoveToPoint, SetFileName, 
      Inquire, GotoLine, Stop, SetupScan, SetupAreaScan
} TCmdId;

// Command-Parameter
typedef enum 
{ 
  LargeSide = 1, SmallSide, ToSmallerAngle, ToLargerAngle, 
      Equidistant, Interpolation, Reflection, LastGoal, Standard, 
      DecreaseWidth, ThisDFPos, BackMove, MaximizeTilt, 
      MaximizeCollimator, MaximizeGradient, DynamicWidth, Result, 
      StaticStepWidth, IncreasePeak, Relative, AreaScanResult, 
      AbsorberUsed, Hwb, Difference, ScanResult, 
      Middle, Array, Start, Peak, Min, Max, List, Argument, Opposite, 
      ForAreaScan, ForScan
} TCParam;

// Macro Bezeichner

typedef enum 
{ 
  InquireHwb = 1, BatchMacro, MiddleOfValley, SearchReflection, 
      Test, AzimutalJustify, SetupTopography, ScanJob, AreaScanJob
} TMacroId;

typedef struct
{
  TCmdId  Id;
  TCParam P1;
  TCParam P2;
  char    P3[51];
} TCmdTag;


typedef struct
{
  TMacroId Id;
  int      Length;
  BOOL     bIsReady;
  TCmdTag* CmdList;
  char     Name[21];
  char     FileName[21];
} TMacroTag;

class TDevice;

class TSteering
{
  friend TIMECALLBACK RecallSteering;
  friend LRESULT CALLBACK FrameWndProc( HWND, UINT, WPARAM, LPARAM );
  friend class TCmd;
  friend class TMacroExecute;
 public:
             TSteering( void );
  BOOL       Initialize( const HWND, const int, const int );
  BOOL       StartUp( const HWND, const int, const int );
  BOOL       Reset( void );
  void       Visualising( int, int, int, int, int, int );
  int        StartCmdExecution( TCmdTag );
  int        StartCmdExecution( TCmdId, int, int, LPSTR, HWND );
  int        StartMacroExecution( TMacroTag*, HWND );
  BOOL       DeviceRequest( void );
  void       SetNorm( float norm ) 
      { fIntensityNorm = norm; };
  float      GetNorm( void )
      { return fIntensityNorm; };
  BOOL       ToggleInterrupt( void );
  int        CallMacro( TMacroTag* );
  float      GetIntensity( void );
  BOOL       IsResponding( void )  
      { return bSendReport; };
  void       SetResponse( BOOL sr ) 
      { bSendReport = sr; };
  double     GetDistance( void );
  void       NotifyCmdReady( void );
  void       NotifyMacroReady( void );
  HWND       GetControlWnd( void )  
      { return hControlWnd; };
  BOOL       IsPositionValid( void );
  void       SetNoiseLevel( float nl ) 
      { fNoiseLevel = nl; };
  //! Neue Funktion, die Verbesserungen bei TMacroExecute erlaubt
  int GetMacroCount( void )
      { return nMacroNumber; }
  TMacroTag* GetMacroByNumber( int );
  TMacroTag* GetMacroById( TMacroId );
  TMacroTag* GetMacroByName( LPSTR );
  BOOL       LoadMacro( LPSTR, LPSTR );
  BOOL       DeleteMacro( TMacroTag* );
  BOOL       LoadMacroByUser( void );
  float      GetSigma( void ) ;     
//      { return Device->GetSigma(); };
  float      GetNoiseLevel( void )   
      { return fNoiseLevel; };
  double     dPeakPoint, dStartPoint;
  float      fPeakIntensity, fStartIntensity, fHWB;
  double     dCalcResult, dCalcArg[nMaxArg];
  double     dGoalDistance;
  float      fGoalIntensity;
  BOOL       bInquireResult;
 protected:
  void       SendReport( void );
  HWND       hReportList;
  //##die vier folgenden Varablen werden im Programmablauf zwar belegt, dann
  //##aber nicht weiter benutzt
  BOOL       b_id_Angle;
  BOOL       b_id_PeakIntensity;
  BOOL       b_id_Intensity;
  BOOL       b_id_HalfWidth;

  BOOL       b_id_Report;
  BOOL       bSendReport;
  //*** Zustands-Flags ****************
  BOOL       bInterrupted;
  TDevice*   Device;
  int        nMotor;
  int        nFailureId;
  TMacroTag  aMacroList[20];
  int        nMacroNumber;
  void       WakeUp( void );
 private:
  char       MessageBuffer[MaxString];
  BOOL       ParsingMacroId( TMacroTag&, LPSTR );
  BOOL       ParsingCmd( TCmdTag&, LPSTR, LPSTR, LPSTR, LPSTR );
  TCParam    ParsingCmdParam( LPSTR param );
  BOOL       GetFileLine( FILE*, LPSTR, int );
  BOOL       ReadyReaction( void );
  int        TimerRequest( void );
  int        ExecuteNextCmd( void );
  BOOL       StartTimer( int );
  BOOL       StopTimer( void );
  BOOL       bTimerActive;
  BOOL       IsMacroCalled( void );
  int        ReturnMacroCall( void );
  float      fIncomingIntensity;
  TMacroTag* TheMacro;
  TCmdTag*   TheCmdList;
  TCmd*      TheCmd;
  int        nCmdIdx;
  BOOL       bSteeringActive;
  float      fIntensityNorm;
  float      fNoiseLevel;
  HWND       hControlWnd;
};


//*****************************************************************************
class TAdjustmentWindow : public TPlotData
{
 public:
         TAdjustmentWindow( void );
         ~TAdjustmentWindow();
 protected:
  BOOL    CanClose( void );
  void    Create( HWND );
  void    InitializeTask( WPARAM, LPARAM );
  void    DrawMenu( void );
  void    rButtonDown( LPARAM );
  void    StopMeasurement( void );
  LPCSTR  ClassName() 
      { return "Adjustment"; };
  LPCSTR  GetCharacteristic() 
      { return "Adjustment"; };
  void    CounterSetRequest( LPARAM );
  void    SteeringReady( LPARAM );
 private:
  BOOL    bContinuousScanActive;
  TCurve* AdjData;
  BOOL    bMacroRunning;
  BOOL    SetTitle( void );
};

class TSetAdjustmentParam : public TModalDlg 
{
 public: 
        TSetAdjustmentParam();
 protected:
  float NoiseLevel;
  float SearchFailure;
 private:
  BOOL  CanClose( void );
  void  SetParameter( void );
  void  GetParameter( void );
  void  Dlg_OnCommand( HWND, int, HWND, UINT );
  BOOL  Dlg_OnInit( HWND, HWND, LPARAM );
  BOOL  LeaveDialog( void );
};


class TMacroExecute : public TModelessDlg
{
 public:      
             TMacroExecute( void );
 private:
  TMacroTag* TheMacro;
  TDevice*   Device;
  HWND       hMacroList;
  BOOL       Dlg_OnInit( HWND, HWND, LPARAM );
  void       Dlg_OnCommand( HWND, int, HWND, UINT );
  BOOL       CanClose( void );
  BOOL       LeaveDialog( void );
  int        MessageLine;
  BOOL       bInterrupted;
  BOOL       bActionCaused;
  int        SetupTask;
  BOOL       bCheckMacro;
};

class TAdjustmentExecute : public TModalDlg
{
 public:      
             TAdjustmentExecute( int );
 private:
  HWND       hMotorList;
  BOOL       Dlg_OnInit( HWND, HWND, LPARAM );
  TMacroTag* GetNextTask( void );
  void       Dlg_OnCommand( HWND, int, HWND, UINT );
  BOOL       CanClose( void );
  void       GetParameter( void );
  BOOL       LeaveDialog( void );
  int        nMotor;
  TDevice*   Device;
  int        MessageLine;
  BOOL       bJustifyActive;
  BOOL       bInterrupt;
  BOOL       bActionCaused;
  int        SetupTask;
  BOOL       bReflectionSearch; 
  BOOL       bAzimutalAdjustment;
  BOOL       bFinalAdjustment;
  BOOL       bGetPeakProperties;
  BOOL       bSearchStartPosition;
  BOOL       bCheckMacro;
};


#endif